@@ -1,3 +1,5 @@
+run_tests <- length(strsplit(as.character(packageVersion("simPop")), "[.]")[[1]]) > 3
+if(run_tests){
 message("sprague")
 ## test sprague index:
 library(simPop)
@@ -17,4 +19,5 @@ x <- data.frame(age=as.factor(c(
 s  <- sprague(x[,2])
 
 expect_true(all.equal(sum(s), sum(x[,2])),info="not identical")
-#
\ No newline at end of file
+#
+}
\ No newline at end of file
